(ns build
  (:require [build.reify2 :as reify2]
            [clojure.tools.build.api :as b]))

(def lib 'org.babashka/babashka.impl.java)
(def version "0.1.8")
(def class-dir "target/classes")
(def basis (b/create-basis {:project "deps.edn"}))
(def jar-file (format "target/%s-%s.jar" (name lib) version))

(defn clean [_]
  (b/delete {:path "target"}))

(defn gen-classes [_]
  (reify2/gen-classes nil))

(defn compile-java [_]
  (b/javac {:src-dirs ["src-java"]
            :class-dir class-dir
            :basis basis}))

(defn jar [_]
  (compile-java nil)
  (gen-classes nil)
  (b/write-pom {:class-dir class-dir
                :lib lib
                :version version
                :basis basis
                :src-dirs ["src"]})
  (b/copy-dir {:src-dirs ["src"]
               :target-dir class-dir})
  (b/jar {:class-dir class-dir
          :jar-file jar-file}))

(defn install [_]
  (jar nil)
  (b/install {:basis basis
              :lib lib
              :version version
              :jar-file jar-file
              :class-dir class-dir}))

(defn deploy [opts]
  (jar opts)
  ((requiring-resolve 'deps-deploy.deps-deploy/deploy)
   (merge {:installer :remote
           :artifact jar-file
           :pom-file (b/pom-path {:lib lib :class-dir class-dir})}
          opts))
  opts)

;;;; Scratch

(comment
  (gen-classes nil)
  )
